/* ***** BEGIN LICENSE BLOCK *****
 * Version: GPL 2.0
 *
 * This code is part of DownThemAll! - dTa!
 * Copyright © 2004-2006 Federico Parodi and Stefano Verna.
 * 
 * See notice.txt and gpl.txt for details.
 *
 * ***** END LICENSE BLOCK ***** */

onerror = function (msg, url, l) {
	var e = {
		message: msg,
		fileName: url,
		lineNumber: l
	}
	Debug.dump("OnError():", e);
	return true;
};

if (!DTA_GUID) const DTA_GUID = '{DDC359D1-844A-42a7-9AA1-88A850A938A8}';

// From prototype.js :)
Object.extend = function(destination, source) {
  for (property in source) {
    destination[property] = source[property];
  }
  return destination;
}

function $(id) {
	return document.getElementById(id);
}

 	
// ProfileFile
// @author Nils Maier <MaierMan@web.de>
// modified version, moves xml to Profile directory
var ProfileFile = {
	_ds : Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties),
	get: function PF_get(fileName)	{
		var file = this._ds.get("ProfD", Components.interfaces.nsIFile)
		file.append(fileName);
		return file;
	}
};


Object.extend(Number.prototype,
{
convertToByte : function() {
	if (this/Math.pow(1024,3) > 1)
		return (this/Math.pow(1024,3)).toFixed(2) + " GB";
	else if (this/Math.pow(1024,2) > 1)
		return (this/Math.pow(1024,2)).toFixed(2) + " MB";
	else if (this/1024 > 1)
		return Math.ceil(this/1024) + " KB";
	else
		return this + " B";
}
}
);

// String
Object.extend(String.prototype, 
{	
	trim : function() {
		return this.replace(/^[ \t_]+|[ \t_]+$/gi, "").replace(/(_){2,}/g, "_");
	},
	removeBadChars : function() {
		return this.replace(/[\?\:<>\*\|"]/g, "_").replace(/%20/g, " ").replace(/%2520/g, " "); //"
	},
	findSystemSlash : function() {
		var path=(ProfileFile.get("dummy")).path;
		if (path.search(/\\/) != -1) return "\\"; else return "/";
	},
	findForbiddenSlash : function() {
		if (this.findSystemSlash() == "/")
			return "\\";
		else
			return "/";
	},
	addFinalSlash : function() {
		if (this.length == 0) return this.findSystemSlash();
		
		if (this[this.length - 1] != this.findSystemSlash())
			return this + this.findSystemSlash();
		else
			return this;
	},
	removeFinalChar : function(c) {
		if (this.length == 0) return this;
		if (this.length == 1) return (this==c)?"":this;
		
		if (this[this.length - 1]==c) {
			return this.substring(0, this.length - 1);
		} else
			return this;
	},
	removeLeadingChar : function(c) {
		if (this.length == 0) return this;
		if (this.length == 1) return (this==c)?"":this;
		
		if (this[0] == c) {
			return this.substring(1, this.length);
		} else
			return this;
	},
	removeFinalSlash : function() {
		return this.removeFinalChar(this.findSystemSlash());
	},
	removeLeadingSlash : function() {
		return this.removeLeadingChar(this.findSystemSlash());
	},
	removeFinalBackSlash : function() {
		return this.removeFinalChar("/");
	},
	removeLeadingBackSlash : function() {
		return this.removeLeadingChar("/");
	},
	removeArguments : function() {
		if (this.indexOf("?") > 0)
			return this.substring(0, this.indexOf("?"));
		else
			return this;
	},
	getUsableFileName : function() {
		var t = this.trim().removeArguments().removeFinalBackSlash().split("/");
		return t[t.length-1].removeBadChars().replace(/[\\/]/g, "").trim();
	},
	getExtension : function() {
		var name = this.getUsableFileName();
		var c = name.split(".");
		if (c.length == 1) 
			return null;
		else
			return c[c.length - 1];
	},
	formatTimeDate : function() {
		return this.replace(/\b(\d)\b/g, "0$1");
	},
	isLinkOpenable : function() {
		return this.match(/^(http|ftp|https):\/\/.+/i)
	}
}
);

// Debug
var Debug = {
	_dumpEnabled : false,
	_consoleService : null,
	_logPointer : null,
	_loaded : false,
	_load : function() {
		this._dumpEnabled = Preferences.get("extensions.dta.directory.visibledump", false);
		this._consoleService = Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService);
		this._logPointer = ProfileFile.get('dta_log.txt');
		try {
			if (this._logPointer.fileSize > (200 * 1024)) 
				this._logPointer.remove(false);
		} catch(e) {}
		this._loaded = true;
	},
	dump : function(message, e) {
	if (!this._loaded) this._load();
	try {
		if (!this._dumpEnabled || (message=="" && typeof(e)!="object")) return;
		var fo = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
		var time = new Date();
		var text = String(time.getHours()).formatTimeDate() + ":" + String(time.getMinutes()).formatTimeDate() + ":" + String(time.getSeconds()).formatTimeDate() + ":" + time.getMilliseconds() + "\x0D\x0A\t";
		
		if (message!="") 
			text += message.replace(/\n/g, "\x0D\x0A\t") + " ";
		if (typeof(e)=="object")
			text += (e.message + " (" + e.fileName +" line " + e.lineNumber + ")");
		
		text += "\x0D\x0A";
		
		this._consoleService.logStringMessage(text);
		
		fo.init(this._logPointer, 0x04 | 0x08 | 0x10, 0664, 0); 
		fo.write(text, text.length);
		fo.close();
	} catch(e) {}
	}
}

// Preferences
var Preferences = {
	_pref : null,
	get : function (stringa, predefinito) {
		if (this._pref==null)
			this._pref = Components.classes['@mozilla.org/preferences-service;1'].getService(Components.interfaces.nsIPrefBranch);
		try {
			if (typeof(predefinito) == "boolean")
				var scelta = this._pref.getBoolPref(stringa);
			else if (typeof(predefinito) == "string")
				var scelta = this._pref.getCharPref(stringa);
			else if (typeof(predefinito) == "number")
				var scelta = this._pref.getIntPref(stringa);
			return scelta;
		} catch (e) {
			if (typeof(predefinito) == "boolean")
				var scelta = this._pref.setBoolPref(stringa, predefinito);
			else if (typeof(predefinito) == "string")
				var scelta = this._pref.setCharPref(stringa, predefinito);
			else
				var scelta = this._pref.setIntPref(stringa, predefinito);
			return predefinito;
		}
	},
	set : function(stringa, value) {
		if (this._pref==null)
			this._pref = Components.classes['@mozilla.org/preferences-service;1'].getService(Components.interfaces.nsIPrefBranch);
			
		if (typeof(value) == "boolean")
			this._pref.setBoolPref(stringa, value);
		else if (typeof(value) == "string")
			this._pref.setCharPref(stringa, value);
		else if (typeof(value) == "number")
			this._pref.setIntPref(stringa, value);
	},
	removeBranch: function(pref) {
		try {
			this._pref.deleteBranch(pref);
		} catch(e) {}
	}
}

// DropdownObject
function dropDownObject(name, idInput, idDropDown, predefined, predefinedHistory) {
	this.name = name;
	this.branch = "extensions.dta.dropdown."+name;
	this.currentValue = this.branch+"-current";
	this.history = this.branch+"-history";
	this.idInput = idInput;
	this.idDropDown = idDropDown;
	this.predefined = predefined;
	this.predefinedHistory = predefinedHistory;
}

dropDownObject.prototype = {

	load : function() {
		var valuesDrop = nsPreferences.getLocalizedUnicharPref(this.history, this.predefinedHistory).split("|@|");
		var drop = $(this.idDropDown);
		var maxDrop = Preferences.get("extensions.dta.context.history", 5);
	
		while (drop.hasChildNodes())
			drop.removeChild(drop.lastChild); 
		
		if (maxDrop>0) {
			var n = 0;
			for (x in valuesDrop) {
				var itemNode = document.createElement("menuitem");
				itemNode.setAttribute("label", valuesDrop[x]);
				drop.appendChild(itemNode);
				if ((++n)==maxDrop) break;
			}
		}
		
		$(this.idInput).value = nsPreferences.getLocalizedUnicharPref(this.currentValue, this.predefined);
	},
	
	getCurrent : function() {
		return $(this.idInput).value;
	},
	
	saveCurrent : function(alsoNothing) {
		nsPreferences.setUnicharPref(this.currentValue, $(this.idInput).value);
	},
	
	saveDrop : function(stringa) {
		if (stringa.length == 0) return;
		var valuesDrop = nsPreferences.getLocalizedUnicharPref(this.history, this.predefinedHistory).split("|@|");
		var maxInDrop = Preferences.get("extensions.dta.context.history", 5);
		
		for (i in valuesDrop)
			if (stringa == valuesDrop[i])
				valuesDrop.splice(i, 1);
	
		if (valuesDrop.length == maxInDrop) valuesDrop.pop();
		valuesDrop.splice(0, 0, stringa);
		
		nsPreferences.setUnicharPref(this.history, valuesDrop.join("|@|"));
		//Preferences.set(this.history, valuesDrop.join("|@|"));
	}
}

function filePicker() {}

filePicker.prototype = {

	getFolder : function (predefined, text) {try {
		// nsIFilePicker object
		var nsIFilePicker = Components.interfaces.nsIFilePicker;
		var fp = Components.classes['@mozilla.org/filepicker;1'].createInstance(nsIFilePicker);
		fp.init(window, text, nsIFilePicker.modeGetFolder);
		fp.appendFilters(nsIFilePicker.filterAll);
		
		// locate current directory
		var dest;
		if ((dest = this.createValidDestination(predefined)))
			fp.displayDirectory = dest;
		
		// open file picker
		var res = fp.show();
	
		if (res == nsIFilePicker.returnOK)
			return fp.file.path.addFinalSlash();
		
	} catch (e) {Debug.dump("filePicker.getFolder():", e);}
	return false;
	},
	
	createValidDestination : function(path) {
			if (!path) return false;
			if (String(path).trim().length==0) return false;
			var directory = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
			
			try {
				directory.initWithPath(path);
				if (directory.exists()) 
					return directory;
			} catch(e) {return false;}
			
			var f = (new String()).findSystemSlash();
			if (f=="/") {
				if ((/[\?\+&=:<>\*\|"\\]/gi).test(path)) return false;
			} else {
				if ((/[\?\+&=:<>\*\|"\/]/gi).test(path.substring(3, path.length))) return false;
			}

		return directory;
	}
	
}
